SystemOrganization addCategory: #'ToolBuilder-Morphic'!

UIManager subclass: #MorphicUIManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!MorphicUIManager commentStamp: 'ar 2/11/2005 21:52' prior: 0!
The Morphic ui manager.!

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 2/28/2005 17:13'!
displayProgress: titleString at: aPoint from: minVal to: maxVal during: workBlock
	"Display titleString as a caption over a progress bar while workBlock is evaluated."
	^ProgressInitiationException 
		display: titleString
		at: aPoint 
		from: minVal 
		to: maxVal 
		during: workBlock! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 08:46'!
inform: aString
	"Display a message for the user to read and then dismiss"
	^PopUpMenu inform: aString! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 10:47'!
chooseFrom: aList lines: linesArray title: aString
	"Choose an item from the given list. Answer the index of the selected item."
	| menu |
	menu := PopUpMenu labelArray: aList lines: linesArray.
	^aString isEmpty ifTrue:[menu startUp] ifFalse:[menu startUpWithCaption: aString]! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 08:46'!
request: queryString initialAnswer: defaultAnswer 
	"Create an instance of me whose question is queryString with the given 
	initial answer. Invoke it centered at the given point, and answer the 
	string the user accepts. Answer the empty string if the user cancels."
	^FillInTheBlank request: queryString initialAnswer: defaultAnswer ! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 08:47'!
requestPassword: queryString
	"Create an instance of me whose question is queryString. Invoke it centered
	at the cursor, and answer the string the user accepts. Answer the empty 
	string if the user cancels."
	^FillInTheBlank requestPassword: queryString! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 2/28/2005 17:07'!
informUserDuring: aBlock
	"Display a message above (or below if insufficient room) the cursor 
	during execution of the given block.
		UIManager default informUserDuring:[:bar|
			#(one two three) do:[:info|
				bar value: info.
				(Delay forSeconds: 1) wait]]"
	(MVCMenuMorph from: (SelectionMenu labels: '') title: '						')
		informUserAt: Sensor cursorPoint during: aBlock.! !

!MorphicUIManager class methodsFor: 'accessing' stamp: 'ar 2/11/2005 15:41'!
isActiveManager
	"Answer whether I should act as the active ui manager"
	^Smalltalk isMorphic! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 08:45'!
confirm: queryString
	"Put up a yes/no menu with caption queryString. Answer true if the 
	response is yes, false if no. This is a modal question--the user must 
	respond yes or no."
	^PopUpMenu confirm: queryString! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 12/27/2004 09:49'!
confirm: aString orCancel: cancelBlock
	"Put up a yes/no/cancel menu with caption aString. Answer true if  
	the response is yes, false if no. If cancel is chosen, evaluate  
	cancelBlock. This is a modal question--the user must respond yes or no."
	^PopUpMenu confirm: aString orCancel: cancelBlock! !

!MorphicUIManager methodsFor: 'ui requests' stamp: 'ar 2/28/2005 17:05'!
multiLineRequest: queryString centerAt: aPoint initialAnswer: defaultAnswer answerHeight: answerHeight
	"Create a multi-line instance of me whose question is queryString with
	the given initial answer. Invoke it centered at the given point, and
	answer the string the user accepts.  Answer nil if the user cancels.  An
	empty string returned means that the ussr cleared the editing area and
	then hit 'accept'.  Because multiple lines are invited, we ask that the user
	use the ENTER key, or (in morphic anyway) hit the 'accept' button, to 
	submit; that way, the return key can be typed to move to the next line."
	^FillInTheBlank multiLineRequest: queryString centerAt: aPoint initialAnswer: defaultAnswer answerHeight: answerHeight! !

AlignmentMorph subclass: #PluggablePanelMorph
	instanceVariableNames: 'model getChildrenSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggablePanelMorph commentStamp: 'ar 2/11/2005 20:13' prior: 0!
A pluggable panel morph which deals with changing children.!

!PluggablePanelMorph methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:48'!
model
	^model! !

!PluggablePanelMorph methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:47'!
getChildrenSelector
	^getChildrenSelector! !

!PluggablePanelMorph methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:47'!
getChildrenSelector: aSymbol
	getChildrenSelector := aSymbol.! !

!PluggablePanelMorph methodsFor: 'update' stamp: 'ar 2/11/2005 20:01'!
update: what
	what == nil ifTrue:[^self].
	what == getChildrenSelector ifTrue:[
		self removeAllMorphs.
		self addAllMorphs: (model perform: getChildrenSelector).
		self submorphsDo:[:m| m hResizing: #spaceFill; vResizing: #spaceFill].
	].! !

!PluggablePanelMorph methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:48'!
model: aModel
	model ifNotNil:[model removeDependent: self].
	model := aModel.
	model ifNotNil:[model addDependent: self].! !

ToolBuilderTests subclass: #MorphicToolBuilderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!MorphicToolBuilderTests commentStamp: 'ar 2/11/2005 15:02' prior: 0!
Tests for the Morphic tool builder.!

!MorphicToolBuilderTests methodsFor: 'tests-window' stamp: 'ar 2/13/2005 13:52'!
testWindowDynamicLabel
	self makeWindow.
	self assert: (widget label = 'TestLabel').! !

!MorphicToolBuilderTests methodsFor: 'support' stamp: 'ar 2/11/2005 19:15'!
fireButtonWidget
	widget performAction.! !

!MorphicToolBuilderTests methodsFor: 'support' stamp: 'ar 2/11/2005 19:26'!
acceptWidgetText
	widget hasUnacceptedEdits: true.
	widget accept.! !

!MorphicToolBuilderTests methodsFor: 'support' stamp: 'cwp 6/9/2005 00:11'!
fireMenuItemWidget
	(widget itemWithWording: 'Menu Item')
		ifNotNilDo: [:item | item doButtonAction]! !

!MorphicToolBuilderTests methodsFor: 'support' stamp: 'ar 2/11/2005 21:43'!
widgetColor
	"Answer color from widget"
	^widget color! !

!MorphicToolBuilderTests methodsFor: 'support' stamp: 'ar 2/11/2005 14:46'!
setUp
	super setUp.
	builder := MorphicToolBuilder new.! !

!MorphicToolBuilderTests methodsFor: 'support' stamp: 'ar 2/11/2005 19:22'!
changeListWidget
	widget changeModelSelection: widget getCurrentSelectionIndex + 1.! !

!MorphicToolBuilderTests methodsFor: 'tests-window' stamp: 'ar 2/13/2005 13:52'!
testWindowStaticLabel
	| spec |
	spec := builder pluggableWindowSpec new.
	spec model: self.
	spec children: #().
	spec label: 'TestLabel'.
	widget := builder build: spec.
	self assert: (widget label = 'TestLabel').! !

PluggableTextMorph subclass: #PluggableTextMorphPlus
	instanceVariableNames: 'getColorSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableTextMorphPlus commentStamp: 'ar 2/11/2005 21:53' prior: 0!
A pluggable text morph with support for color.!

!PluggableTextMorphPlus methodsFor: 'updating' stamp: 'ar 2/11/2005 21:22'!
update: what
	what ifNil:[^self].
	what == getColorSelector ifTrue:[self color: (model perform: getColorSelector)].
	^super update: what! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 21:21'!
getColorSelector
	^getColorSelector! !

!PluggableTextMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 21:21'!
getColorSelector: aSymbol
	getColorSelector := aSymbol.
	self update: getColorSelector.! !

SystemWindow subclass: #PluggableSystemWindow
	instanceVariableNames: 'getLabelSelector getChildrenSelector children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableSystemWindow commentStamp: 'ar 2/11/2005 20:14' prior: 0!
A pluggable system window. Fixes the issues with label retrieval and adds support for changing children.!

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:57'!
getChildrenSelector
	^getChildrenSelector! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/13/2005 13:52'!
label
	^label contents! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:57'!
getLabelSelector
	^getLabelSelector! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:57'!
getChildrenSelector: aSymbol
	getChildrenSelector := aSymbol! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/13/2005 13:51'!
label: aString
	self setLabel: aString.! !

!PluggableSystemWindow methodsFor: 'accessing' stamp: 'ar 2/13/2005 13:53'!
getLabelSelector: aSymbol
	getLabelSelector := aSymbol.
	self update: aSymbol.! !

!PluggableSystemWindow methodsFor: 'updating' stamp: 'ar 2/11/2005 20:15'!
update: what
	what ifNil:[^self].
	what == getLabelSelector ifTrue:[self setLabel: (model perform: getLabelSelector)].
	what == getChildrenSelector ifTrue:[
		children ifNil:[children := #()].
		self removeAllMorphsIn: children.
		children := model perform: getChildrenSelector.
		self addAllMorphs: children.
		children do:[:m| m hResizing: #spaceFill; vResizing: #spaceFill].
	].
	^super update: what! !

SimpleHierarchicalListMorph subclass: #PluggableTreeMorph
	instanceVariableNames: 'roots selectedWrapper getRootsSelector getChildrenSelector hasChildrenSelector getLabelSelector getIconSelector getSelectedPathSelector setSelectedSelector getHelpSelector dropItemSelector wantsDropSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableTreeMorph commentStamp: 'ar 2/12/2005 04:38' prior: 0!
A pluggable tree morph.!

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:57'!
getMenuSelector
	^getMenuSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:58'!
keystrokeActionSelector: aSymbol
	keystrokeActionSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:11'!
hasNodeContents: node
	hasChildrenSelector ifNil:[^node contents isEmpty not].
	^model perform: hasChildrenSelector with: node item! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
getHelpSelector: aSymbol
	getHelpSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 01:13'!
contentsOfNode: node
	| children |
	getChildrenSelector ifNil:[^#()].
	children := model perform: getChildrenSelector with: node item.
	^children collect:[:item| PluggableTreeItemNode with: item model: self]! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:02'!
isDraggableNode: node
	^true! !

!PluggableTreeMorph methodsFor: 'selection' stamp: 'ar 2/12/2005 01:20'!
setSelectedMorph: aMorph
	selectedWrapper := aMorph complexContents.
	self selection: selectedWrapper.
	setSelectedSelector ifNotNil:[
		model 
			perform: setSelectedSelector 
			with: (selectedWrapper ifNotNil:[selectedWrapper item]).
	].! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:24'!
getChildrenSelector
	^getChildrenSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
getHelpSelector
	^getHelpSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
setSelectedSelector
	^setSelectedSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
dropItemSelector: aSymbol
	dropItemSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
setSelectedSelector: aSymbol
	setSelectedSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:57'!
getMenuSelector: aSymbol
	getMenuSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:19'!
balloonTextForNode: node
	getHelpSelector ifNil:[^nil].
	^model perform: getHelpSelector with: node item! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:25'!
getLabelSelector: aSymbol
	getLabelSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:24'!
getChildrenSelector: aSymbol
	getChildrenSelector := aSymbol.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:58'!
keystrokeActionSelector
	^keystrokeActionSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:27'!
wantsDropSelector: aSymbol
	wantsDropSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:25'!
getIconSelector
	^getIconSelector! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:20'!
dropNode: srcNode on: dstNode
	dropItemSelector ifNil:[^nil].
	model perform: dropItemSelector with: srcNode item with: dstNode item! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
wantsDropSelector
	^wantsDropSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:24'!
hasChildrenSelector
	^hasChildrenSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:25'!
hasChildrenSelector: aSymbol
	hasChildrenSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'updating' stamp: 'ar 2/12/2005 17:29'!
selectPath: path in: listItem
	path isEmpty ifTrue: [^self setSelectedMorph: nil].
	listItem withSiblingsDo: [:each | 
		(each complexContents item = path first) ifTrue: [
			each isExpanded ifFalse: [
				each toggleExpandedState.
				self adjustSubmorphPositions.
			].
			each changed.
			path size = 1 ifTrue: [
				^self setSelectedMorph: each
			].
			each firstChild ifNil: [^self setSelectedMorph: nil].
			^self selectPath: path allButFirst in: each firstChild
		].
	].
	^self setSelectedMorph: nil

! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:25'!
getIconSelector: aSymbol
	getIconSelector := aSymbol! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:24'!
getRootsSelector
	^getRootsSelector! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:21'!
wantsDroppedNode: srcNode on: dstNode
	wantsDropSelector ifNil:[^false].
	model perform: wantsDropSelector with: srcNode with: dstNode.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:24'!
getRootsSelector: aSelector
	getRootsSelector := aSelector.
	self update: getRootsSelector.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:26'!
dropItemSelector
	^dropItemSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 01:11'!
roots: anArray
	roots := anArray collect:[:item| PluggableTreeItemNode with: item model: self].
	self list: roots.! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:25'!
getLabelSelector
	^getLabelSelector! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:20'!
iconOfNode: node
	getIconSelector ifNil:[^nil].
	^model perform: getIconSelector with: node item! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 03:33'!
getSelectedPathSelector
	^getSelectedPathSelector! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 03:33'!
getSelectedPathSelector: aSymbol
	getSelectedPathSelector := aSymbol.! !

!PluggableTreeMorph methodsFor: 'node access' stamp: 'ar 2/12/2005 00:20'!
printNode: node
	getLabelSelector ifNil:[^node item printString].
	^model perform: getLabelSelector with: node item! !

!PluggableTreeMorph methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:22'!
roots
	^roots! !

!PluggableTreeMorph methodsFor: 'updating' stamp: 'ar 2/12/2005 19:11'!
update: what
	what ifNil:[^self].
	what == getRootsSelector ifTrue:[
		self roots: (model perform: getRootsSelector)
	].
	what == getSelectedPathSelector ifTrue:[
		^self selectPath: (model perform: getSelectedPathSelector)
			in: (scroller submorphs at: 1 ifAbsent: [^self]) 
	].
	^super update: what! !

ListItemWrapper subclass: #PluggableTreeItemNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableTreeItemNode commentStamp: 'ar 2/12/2005 04:37' prior: 0!
Tree item for PluggableTreeMorph.!

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:16'!
wantsDroppedObject: anotherItem
	^model wantsDroppedNode: anotherItem on: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 01:00'!
item
	^item! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:16'!
asString
	^model printNode: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:03'!
contents
	^model contentsOfNode: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:16'!
hasContents
	^model hasNodeContents: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:02'!
canBeDragged
	^model isDraggableNode: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:16'!
acceptDroppingObject: anotherItem
	^model dropNode: anotherItem on: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:04'!
icon
	^model iconOfNode: self! !

!PluggableTreeItemNode methodsFor: 'accessing' stamp: 'ar 2/12/2005 00:16'!
balloonText
	^model balloonTextForNode: self! !

PluggableButtonMorph subclass: #PluggableButtonMorphPlus
	instanceVariableNames: 'enabled action getColorSelector getEnabledSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!PluggableButtonMorphPlus commentStamp: 'ar 2/11/2005 21:53' prior: 0!
An extended version of PluggableButtonMorph supporting enablement, color and block/message actions.!

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:51'!
getEnabledSelector: aSymbol
	getEnabledSelector := aSymbol.! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 20:55'!
action: anAction	
	action := nil.
	anAction isSymbol ifTrue:[^super action: anAction].
	action := anAction.! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/13/2005 13:56'!
enabled: aBool
	enabled := aBool.
	enabled 
		ifTrue:[self color: Color gray]
		ifFalse:[self getModelState
			ifTrue: [self color: onColor]
			ifFalse: [self color: offColor]]! !

!PluggableButtonMorphPlus methodsFor: 'action' stamp: 'ar 2/11/2005 19:53'!
mouseUp: evt
	enabled ifFalse:[^self].
	^super mouseUp: evt! !

!PluggableButtonMorphPlus methodsFor: 'updating' stamp: 'ar 2/11/2005 21:39'!
update: what
	what ifNil:[^self].
	what == getLabelSelector ifTrue: [
		self label: (model perform: getLabelSelector)].
	what == getColorSelector ifTrue: [
		color := (model perform: getColorSelector).
		self onColor: color offColor: color.
		self changed.
	].
	what == getStateSelector ifTrue:[
		self getModelState
			ifTrue: [self color: onColor]
			ifFalse: [self color: offColor].
	].
	what == getEnabledSelector ifTrue:[^self enabled: (model perform: getEnabledSelector)].! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 21:36'!
getColorSelector
	^getColorSelector! !

!PluggableButtonMorphPlus methodsFor: 'action' stamp: 'ar 2/11/2005 19:52'!
mouseDown: evt
	enabled ifFalse:[^self].
	^super mouseDown: evt! !

!PluggableButtonMorphPlus methodsFor: 'action' stamp: 'ar 2/11/2005 19:52'!
mouseMove: evt
	enabled ifFalse:[^self].
	^super mouseMove: evt! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:51'!
getEnabledSelector
	^getEnabledSelector! !

!PluggableButtonMorphPlus methodsFor: 'action' stamp: 'ar 2/11/2005 20:54'!
performAction
	enabled ifFalse:[^self].
	action ifNotNil:[^action value].
	^super performAction! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 21:36'!
getColorSelector: aSymbol
	getColorSelector := aSymbol.
	self update: getColorSelector.! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 20:53'!
action
	^action! !

!PluggableButtonMorphPlus methodsFor: 'initialize-release' stamp: 'ar 2/11/2005 21:56'!
initialize
	super initialize.
	enabled := true.
	self color: Color transparent.! !

!PluggableButtonMorphPlus methodsFor: 'initialize-release' stamp: 'ar 2/12/2005 14:23'!
beActionButton
	"Make me like an action button"
	self borderWidth: 2.
	self borderColor: #raised.
	self onColor: Color transparent offColor: Color transparent.
	self cornerStyle: #rounded.! !

!PluggableButtonMorphPlus methodsFor: 'accessing' stamp: 'ar 2/11/2005 19:52'!
enabled
	^enabled! !

ToolBuilder subclass: #MorphicToolBuilder
	instanceVariableNames: 'panes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!MorphicToolBuilder commentStamp: 'ar 2/11/2005 15:02' prior: 0!
The Morphic tool builder.!

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 2/11/2005 21:32'!
buildPluggableButton: aSpec
	| widget label state action |
	label := aSpec label.
	state := aSpec state.
	action := aSpec action.
	widget := PluggableButtonMorphPlus on: aSpec model
				getState: (state isSymbol ifTrue:[state])
				action: nil
				label: (label isSymbol ifTrue:[label]).
	widget action: action.
	widget getColorSelector: aSpec color.
	widget offColor: Color transparent.
	widget getEnabledSelector: aSpec enabled.
	aSpec help ifNotNil:[widget setBalloonText: aSpec help].
	(label isSymbol or:[label == nil]) ifFalse:[widget label: label].
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[parent addMorphBack: widget].
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 2/12/2005 14:22'!
buildPluggableActionButton: aSpec
	| button |
	button := self buildPluggableButton: aSpec.
	button beActionButton.
	^button! !

!MorphicToolBuilder class methodsFor: 'accessing' stamp: 'ar 2/11/2005 15:24'!
isActiveBuilder
	"Answer whether I am the currently active builder"
	^Smalltalk isMorphic! !

!MorphicToolBuilder methodsFor: 'building' stamp: 'ar 2/12/2005 01:53'!
open: anObject label: aString
	| window |
	window := self open: anObject.
	window setLabel: aString.
	^window! !

!MorphicToolBuilder methodsFor: 'building' stamp: 'ar 2/12/2005 18:36'!
open: anObject
	| window |
	window := self build: anObject.
	window openInWorldExtent: window extent.
	^window! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 2/12/2005 16:42'!
buildPluggableList: aSpec
	| widget listClass getIndex setIndex |
	aSpec getSelected ifNil:[
		listClass := PluggableListMorph.
		getIndex := aSpec getIndex.
		setIndex := aSpec setIndex.
	] ifNotNil:[
		listClass := PluggableListMorphByItem.
		getIndex := aSpec getSelected.
		setIndex := aSpec setSelected.
	].
	widget := listClass on: aSpec model
				list: aSpec list
				selected: getIndex
				changeSelected: setIndex
				menu: aSpec menu
				keystroke: aSpec keyPress.
	widget autoDeselect: aSpec autoDeselect.
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[parent addMorphBack: widget].
	panes ifNotNil:[
		aSpec list ifNotNil:[panes add: aSpec list].
	].
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 2/11/2005 18:38'!
buildPluggableInputField: aSpec
	| widget |
	widget := self buildPluggableText: aSpec.
	widget acceptOnCR: true.
	widget hideScrollBarIndefinitely.
	^widget! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'ar 2/9/2005 18:56'!
asFrame: aRectangle
	| frame |
	aRectangle ifNil:[^nil].
	frame _ LayoutFrame new.
	frame 
		leftFraction: aRectangle left; 
		rightFraction: aRectangle right; 
		topFraction: aRectangle top; 
		bottomFraction: aRectangle bottom.
	^frame! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 2/12/2005 16:43'!
buildPluggableTree: aSpec
	| widget |
	widget := PluggableTreeMorph new.
	widget model: aSpec model.
	widget getSelectedPathSelector: aSpec getSelectedPath.
	widget setSelectedSelector: aSpec setSelected.
	widget getChildrenSelector: aSpec getChildren.
	widget hasChildrenSelector: aSpec hasChildren.
	widget getLabelSelector: aSpec label.
	widget getIconSelector: aSpec icon.
	widget getHelpSelector: aSpec help.
	widget getMenuSelector: aSpec menu.
	widget keystrokeActionSelector: aSpec keyPress.
	widget getRootsSelector: aSpec roots.
	widget autoDeselect: aSpec autoDeselect.
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[parent addMorphBack: widget].
	panes ifNotNil:[
		aSpec roots ifNotNil:[panes add: aSpec roots].
	].
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 2/11/2005 21:23'!
buildPluggableText: aSpec
	| widget |
	widget := PluggableTextMorphPlus on: aSpec model
				text: aSpec getText 
				accept: aSpec setText
				readSelection: aSpec selection 
				menu: aSpec menu.
	widget getColorSelector: aSpec color.
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[parent addMorphBack: widget].
	panes ifNotNil:[
		aSpec getText ifNotNil:[panes add: aSpec getText].
	].
	^widget! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'ar 2/12/2005 19:20'!
setFrame: aRectangle in: widget
	| frame |
	aRectangle ifNil:[^nil].
	frame := self asFrame: aRectangle.
	widget layoutFrame: frame.
	widget hResizing: #spaceFill; vResizing: #spaceFill.
	(parent isSystemWindow) ifTrue:[
		widget borderWidth: 2; borderColor: #inset.
	].! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'ar 2/10/2005 22:28'!
setLayout: layout in: widget
	layout == #proportional ifTrue:[
		widget layoutPolicy: ProportionalLayout new.
		^self].
	layout == #horizontal ifTrue:[
		widget layoutPolicy: TableLayout new.
		widget listDirection: #leftToRight.
		widget submorphsDo:[:m| m hResizing: #spaceFill; vResizing: #spaceFill].
		"and then some..."
		^self].
	layout == #vertical ifTrue:[
		widget layoutPolicy: TableLayout new.
		widget listDirection: #topToBottom.
		widget submorphsDo:[:m| m hResizing: #spaceFill; vResizing: #spaceFill].
		"and then some..."
		^self].
	^self error: 'Unknown layout: ', layout.! !

!MorphicToolBuilder methodsFor: 'building' stamp: 'cwp 6/8/2005 23:57'!
buildPluggableMenu: aMenuSpec 
	| menu |
	menu := MenuMorph new.
	aMenuSpec items keysAndValuesDo:
		[:label :message |
		menu
			add: label
			target: message receiver
			selector: message selector
			argumentList: message arguments].
	^ menu! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 2/11/2005 20:43'!
buildPluggablePanel: aSpec
	| widget children |
	widget := PluggablePanelMorph new.
	widget model: aSpec model.
	widget color: Color transparent.
	widget clipSubmorphs: true.
	children := aSpec children.
	children isSymbol ifTrue:[
		widget getChildrenSelector: children.
		widget update: children.
		children := #().
	].
	self buildAll: children in: widget.
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[parent addMorphBack: widget].
	self setLayout: aSpec layout in: widget.
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 2/12/2005 14:15'!
buildPluggableMultiSelectionList: aSpec
	| widget listClass |
	aSpec getSelected ifNotNil:[^self error:'There is no PluggableListMorphOfManyByItem'].
	listClass := PluggableListMorphOfMany.
	widget := listClass on: aSpec model
		list: aSpec list
		primarySelection: aSpec getIndex
		changePrimarySelection: aSpec setIndex
		listSelection: aSpec getSelectionList
		changeListSelection: aSpec setSelectionList
		menu: aSpec menu.
	widget keystrokeActionSelector: aSpec keyPress.
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[parent addMorphBack: widget].
	panes ifNotNil:[
		aSpec list ifNotNil:[panes add: aSpec list].
	].
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 2/13/2005 13:48'!
buildPluggableWindow: aSpec
	| widget children label |
	aSpec layout == #proportional ifFalse:[
		"This needs to be implemented - probably by adding a single pane and then the rest"
		^self error: 'Not implemented'.
	].
	widget := PluggableSystemWindow new.
	widget model: aSpec model.
	(label := aSpec label) ifNotNil:[
		label isSymbol 
			ifTrue:[widget getLabelSelector: label]
			ifFalse:[widget setLabel: label]].
	children := aSpec children.
	children isSymbol ifTrue:[
		widget getChildrenSelector: children.
		widget update: children.
		children := #().
	].
	panes := OrderedCollection new.
	self buildAll: children in: widget.
	aSpec extent ifNotNil:[widget extent: aSpec extent].
	widget setUpdatablePanesFrom: panes.
	^widget! !
